#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file RTC_GameController_Python.py
 @brief PS4 Controller in Python
 @date $Date$


"""
import sys
import time
sys.path.append(".")

# Import RTM module
import RTC
import OpenRTM_aist

import pygame
import os
import pprint

# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
rtc_gamecontroller_python_spec = ["implementation_id", "RTC_GameController_Python", 
		 "type_name",         "RTC_GameController_Python", 
		 "description",       "PS4 Controller in Python", 
		 "version",           "1.0.0", 
		 "vendor",            "Fsk", 
		 "category",          "PS4 Controlle", 
		 "activity_type",     "STATIC", 
		 "max_instance",      "1", 
		 "language",          "Python", 
		 "lang_type",         "SCRIPT",
		 ""]
# </rtc-template>

##
# @class RTC_GameController_Python
# @brief PS4 Controller in Python
# 
# 
class RTC_GameController_Python(OpenRTM_aist.DataFlowComponentBase):
	
	##
	# @brief constructor
	# @param manager Maneger Object
	# 
	def __init__(self, manager):
		OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)


		self._d_analog = RTC.TimedDoubleSeq(RTC.Time(0,0),[])
		"""
		コントローラのアナログ情報出力
		"""
		self._AnalogOut = OpenRTM_aist.OutPort("Analog", self._d_analog)

		self._d_button = RTC.TimedULong(RTC.Time(0,0),0)
		"""
		取得したボタンデータを１６進数で出力
		"""
		self._ButtonOut = OpenRTM_aist.OutPort("Button", self._d_button)

		self._d_conttoller_type = RTC.TimedString(RTC.Time(0,0),"")
		"""
		コントローラのデバイス固有名
		"""
		self._Controller_TypeOut = OpenRTM_aist.OutPort("Controller_Type", self._d_conttoller_type)


		


		# initialize of configuration-data.
		# <rtc-template block="init_conf_param">
		
		# </rtc-template>


		 
	##
	#
	# The initialize action (on CREATED->ALIVE transition)
	# formaer rtc_init_entry() 
	# 
	# @return RTC::ReturnCode_t
	# 
	#
	def onInitialize(self):
		# Bind variables and configuration variable
		
		# Set InPort buffers
		
		# Set OutPort buffers
		self.addOutPort("Analog",self._AnalogOut)
		self.addOutPort("Button",self._ButtonOut)
		self.addOutPort("Controller_Type",self._Controller_TypeOut)
		
		# Set service provider to Ports
		
		# Set service consumers to Ports
		
		# Set CORBA Service Ports
		
		# PS4Contorller Init
		pygame.init()
		pygame.joystick.init()
		self.controller = pygame.joystick.Joystick(0)
		self.controller.init()
		self.err =0
		
		self.MinStick=0.2
		
		self.axis_data ={}
		for i in range(6):
			self.axis_data[i]=0
		
		self.button_data = {}
		for i in range(self.controller.get_numbuttons()):
		    self.button_data[i] = False
		
		
		self.hat_data = {}
		for i in range(self.controller.get_numhats()):
			self.hat_data[i] = (0, 0)
		
		self._d_conttoller_type.data = self.controller.get_name()
		print"device name:"+str(self._d_conttoller_type.data)
		
		return RTC.RTC_OK
	
	#	##
	#	# 
	#	# The finalize action (on ALIVE->END transition)
	#	# formaer rtc_exiting_entry()
	#	# 
	#	# @return RTC::ReturnCode_t
	#
	#	# 
	#def onFinalize(self):
	#
	#	return RTC.RTC_OK
	
	#	##
	#	#
	#	# The startup action when ExecutionContext startup
	#	# former rtc_starting_entry()
	#	# 
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#
	#	#
	#def onStartup(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#	##
	#	#
	#	# The shutdown action when ExecutionContext stop
	#	# former rtc_stopping_entry()
	#	#
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#
	#	#
	#def onShutdown(self, ec_id):
	#
	#	return RTC.RTC_OK
	
		##
		#
		# The activated action (Active state entry action)
		# former rtc_active_entry()
		#
		# @param ec_id target ExecutionContext Id
		# 
		# @return RTC::ReturnCode_t
		#
		#
	def onActivated(self, ec_id):
		print "Active"
		self.err =1
		return RTC.RTC_OK
	
		##
		#
		# The deactivated action (Active state exit action)
		# former rtc_active_exit()
		#
		# @param ec_id target ExecutionContext Id
		#
		# @return RTC::ReturnCode_t
		#
		#
	def onDeactivated(self, ec_id):
		print "Deactive"
		return RTC.RTC_OK
	
		##
		#
		# The execution action that is invoked periodically
		# former rtc_active_do()
		#
		# @param ec_id target ExecutionContext Id
		#
		# @return RTC::ReturnCode_t
		#
		#
	def onExecute(self, ec_id):
		self.err =2
		
		
		for event in pygame.event.get():
			self.err =3
			if event.type == pygame.JOYAXISMOTION:
				self.err =3.1
				self.axis_data[event.axis] = round(event.value,2)
			elif event.type == pygame.JOYBUTTONDOWN:
				self.err =3.2
				self.button_data[event.button] = True
			elif event.type == pygame.JOYBUTTONUP:
				self.err =3.3
				self.button_data[event.button] = False
			elif event.type == pygame.JOYHATMOTION:
				self.err =3.4
				self.hat_data[event.hat] = event.value

		self.err =4

		self.err =6
		binary =1
		self._d_button.data=0
		for i in self.button_data:
			binary << i
			if self.button_data[i] == True:
				self._d_button.data += binary << i 
		self._ButtonOut.write()

		self.err =7
		analogSeq = []
		analogSeq.append(float(self.hat_data[0][1]))  # HAT Up Down
		analogSeq.append(float(self.hat_data[0][0]))  # HAT Lefft Right
		self.err =7.1
		for i in range(4):
			if self.axis_data[i] <self.MinStick and self.axis_data[i] > -self.MinStick:
				self.axis_data[i]=0
		
		analogSeq.append(self.axis_data[1])  # Left Stick Up Down
		analogSeq.append(self.axis_data[0])  # Left Stick Lefft Right
		analogSeq.append(self.axis_data[3])  # Right Stick Up Down
		analogSeq.append(self.axis_data[2])  # Right Stick Lefft Right
		self.err =7.2
		analogSeq.append(self.axis_data[4])  # L2 Button
		analogSeq.append(self.axis_data[5])  # R2 Button
		self._d_analog.data =analogSeq
		self._AnalogOut.write()
		self.err =8
		
		self._Controller_TypeOut.write()
		
		pprint.pprint(self.button_data)
		pprint.pprint(self._d_button.data)
		pprint.pprint(self.axis_data)
		pprint.pprint(self.hat_data)
		
		return RTC.RTC_OK
	
	#	##
	#	#
	#	# The aborting action when main logic error occurred.
	#	# former rtc_aborting_entry()
	#	#
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#
	#	#
	#def onAborting(self, ec_id):
	#
	#	return RTC.RTC_OK
	
		##
		#
		# The error action in ERROR state
		# former rtc_error_do()
		#
		# @param ec_id target ExecutionContext Id
		#
		# @return RTC::ReturnCode_t
		#
		#
	def onError(self, ec_id):
		print "err:"+str(self.err)
		pprint.pprint(self.button_data)
		pprint.pprint(self._d_button.data)
		pprint.pprint(self.axis_data)
		pprint.pprint(self.hat_data)
		return RTC.RTC_OK
	
	#	##
	#	#
	#	# The reset action that is invoked resetting
	#	# This is same but different the former rtc_init_entry()
	#	#
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#
	#	#
	#def onReset(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#	##
	#	#
	#	# The state update action that is invoked after onExecute() action
	#	# no corresponding operation exists in OpenRTm-aist-0.2.0
	#	#
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#

	#	#
	#def onStateUpdate(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#	##
	#	#
	#	# The action that is invoked when execution context's rate is changed
	#	# no corresponding operation exists in OpenRTm-aist-0.2.0
	#	#
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#
	#	#
	#def onRateChanged(self, ec_id):
	#
	#	return RTC.RTC_OK
	



def RTC_GameController_PythonInit(manager):
    profile = OpenRTM_aist.Properties(defaults_str=rtc_gamecontroller_python_spec)
    manager.registerFactory(profile,
                            RTC_GameController_Python,
                            OpenRTM_aist.Delete)

def MyModuleInit(manager):
    RTC_GameController_PythonInit(manager)

    # Create a component
    comp = manager.createComponent("RTC_GameController_Python")

def main():
	mgr = OpenRTM_aist.Manager.init(sys.argv)
	mgr.setModuleInitProc(MyModuleInit)
	mgr.activateManager()
	mgr.runManager()

if __name__ == "__main__":
	main()

